@@ -94,6 +94,7 @@ enum connman_service_connect_reason {
 	CONNMAN_SERVICE_CONNECT_REASON_AUTO	= 1,
 	CONNMAN_SERVICE_CONNECT_REASON_USER	= 2,
 	CONNMAN_SERVICE_CONNECT_REASON_SESSION	= 3,
+	CONNMAN_SERVICE_CONNECT_REASON_NATIVE	= 4,
 };
 
 struct connman_service;
@@ -114,9 +115,12 @@ void connman_service_unref_debug(struct connman_service *service,
 			const char *file, int line, const char *caller);
 
 enum connman_service_type connman_service_get_type(struct connman_service *service);
+enum connman_service_state connman_service_get_state(struct connman_service *service);
 char *connman_service_get_interface(struct connman_service *service);
 
+const char *connman_service_get_identifier(struct connman_service *service);
 const char *connman_service_get_domainname(struct connman_service *service);
+const char *connman_service_get_dbuspath(struct connman_service *service);
 char **connman_service_get_nameservers(struct connman_service *service);
 char **connman_service_get_timeservers_config(struct connman_service *service);
 char **connman_service_get_timeservers(struct connman_service *service);
@@ -128,7 +132,16 @@ const char *connman_service_get_proxy_url(struct connman_service *service);
 const char *connman_service_get_proxy_autoconfig(struct connman_service *service);
 bool connman_service_get_favorite(struct connman_service *service);
 bool connman_service_get_autoconnect(struct connman_service *service);
+bool connman_service_set_autoconnect(struct connman_service *service,
+							bool autoconnect);
 
+/* Return non-zero value to terminate the loop, zero to continue */
+typedef int (* connman_service_iterate_cb) (struct connman_service *service,
+							void *user_data);
+int connman_service_iterate_services(connman_service_iterate_cb cb,
+							void *user_data);
+
+struct connman_service *connman_service_get_default(void);
 struct connman_service *connman_service_lookup_from_network(struct connman_network *network);
 struct connman_service *connman_service_lookup_from_identifier(const char* identifier);
 
