@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2012  BMW Car IT GmbH. All rights reserved.
+ *  Copyright (C) 2012-2014  BMW Car IT GmbH.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -488,7 +488,7 @@ static void extract_settings(DBusMessageIter *array,
 	if (index < 0)
 		goto out;
 
-	info->address = connman_ipaddress_alloc(CONNMAN_IPCONFIG_TYPE_IPV4);
+	info->address = connman_ipaddress_alloc(AF_INET);
 	if (!info->address)
 		goto out;
 
