@@ -40,6 +40,7 @@
 #include <connman/dbus.h>
 #include <connman/log.h>
 #include <connman/technology.h>
+#include <connman/setting.h>
 
 #include "mcc.h"
 
@@ -910,7 +911,7 @@ static void extract_ipv4_settings(DBusMessageIter *array,
 	if (context->ipv4_method != CONNMAN_IPCONFIG_METHOD_FIXED)
 		goto out;
 
-	context->ipv4_address = connman_ipaddress_alloc(CONNMAN_IPCONFIG_TYPE_IPV4);
+	context->ipv4_address = connman_ipaddress_alloc(AF_INET);
 	if (!context->ipv4_address) {
 		context->index = -1;
 		goto out;
@@ -998,7 +999,7 @@ static void extract_ipv6_settings(DBusMessageIter *array,
 	context->ipv6_method = CONNMAN_IPCONFIG_METHOD_AUTO;
 
 	context->ipv6_address =
-		connman_ipaddress_alloc(CONNMAN_IPCONFIG_TYPE_IPV6);
+		connman_ipaddress_alloc(AF_INET6);
 	if (!context->ipv6_address)
 		goto out;
 
@@ -1710,6 +1711,10 @@ static void netreg_update_regdom(struct modem_data *modem,
 	char *alpha2;
 	int mcc;
 
+	/* Do not change regdom here if it is set to follow timezone. */
+	if (connman_setting_get_bool("RegdomFollowsTimezone"))
+		return;
+
 	dbus_message_iter_get_basic(value, &mobile_country_code);
 
 	DBG("%s MobileContryCode %s", modem->path, mobile_country_code);
